;In order to define objc classes you need the ns package and
; that is why you need to require the "COCOA" module at compile time.
(eval-when (:compile-toplevel :load-toplevel :execute)
  (require "COCOA"))

(defun c2f (c)
  "Convert centigrade measurements to farenheit"
  (+ 32.0 (* c (/ 9 5))))

(defun f2c (f)
  "Convert farenheit measurements to centigrades"
  (/ (- f 32.0) (/ 9 5)))

; Define the pathname of the nibfile
(defparameter *temperatures-nibfile* 
  (merge-pathnames #p"Temperatures.nib" *load-pathname*))

; Here the eval-when is needed since the class needs to be created to define
; objc methods and objc methods needs to be defined before you can compile a
; send to them. 
(eval-when (:compile-toplevel :load-toplevel :execute)

  (defclass temperature-converter (ns:ns-window-controller)
    ((centigrade :foreign-type :id :reader centigrade)
     (farenheit :foreign-type :id :reader farenheit))
    (:metaclass ns:+ns-object) ; Needed to make an objective-c class
    (:documentation 
     "A cocoa class that controls the temperature converter and its gui"))

; Define two callbacks that are called by cocoa when the textfields for the
; temperature looses focus the first one is inside the eval-when since it is
; called from another method

  (ccl::define-objc-method ((:void :convert-centigrade-to-farenheit (:id sender))
                            temperature-converter)
    (declare (ignore sender))
    (ccl::send (farenheit self) 
               :set-float-value (c2f (ccl::send (centigrade self) 'float-value)))))

(ccl::define-objc-method ((:void :convert-farenheit-to-centigrade (:id sender))
                          temperature-converter)
  (declare (ignore sender))
  (ccl::send (centigrade self)
             :set-float-value (f2c (ccl::send (farenheit self) 'float-value))))

(ccl::define-objc-method ((:id init) temperature-converter)
  ; Make the temperature-converter use the nib-file
  (ccl::send-super :init-with-window-nib-path (ccl::%make-nsstring (namestring (truename *temperatures-nibfile*))) 
                   :owner self)
  self)

(ccl::define-objc-method ((:void window-did-load) temperature-converter)
  ; Make sure the temperature fields contain the same temperature 
  ; (but in different units)
  (ccl::send self :convert-centigrade-to-farenheit self))

(ccl::define-objc-method ((:void window-will-close) temperature-converter)
  ; Autorelease our window controller to bring down the gui cleanly when the
  ; window is closed                       
  (ccl::send self 'autorelease))

(defun open-gui ()
  "Create and show an instance of the temperature converter"
  (ccl::with-autorelease-pool
   (ccl::send (make-instance 'temperature-converter) :show-window nil)))


